#################################################
#  Plot string data that is 
#  in the stringdata class
#  takes the data stringdata, and a (random)
#  ordering variable - if you want to plot the 
#  data by fathers or mothers
plot.parentagedata <- function(data,ordering,...)
{
    d <- generate.alldata(data);
    plot(1,1,xlim=c(0,1),ylim=c(0,1),type="n",bty="n",xaxt="n"
	    ,yaxt="n",xlab="",ylab="",...);
    if (missing(ordering)) dat <- d$data
    else dat <- d$data[ordering,];
  
    plot.factor.grid(dat,d$cols
                     ,rep(1:length(d$alleles),rep(2,length(d$alleles)))
                     ,xlim=c(0,0.5),ylim=c(0,.98),add=T)
    plot.freqrows(d$p,d$cols[,-1],d$alleles,xlim=c(0.51,1)
                  ,ylim=c(0,0.5),newplot=F,trans=T)
    text(0.75,0.54,"Background Frequencies",cex=1.2);
    text(0.25,1.02,"Locus",cex=1.2);
    l <- length(d$alleles);
    text(seq(0.5/(2*l),0.5-0.5/(2*l),0.5/l),rep(0.995,l),1:l,cex=0.8);
    text(0.75,1.02,"Key",cex=1.2);
    text(seq(0.5+0.5/(2*l),1-0.5/(2*l),0.5/l),rep(0.995,l),1:l,cex=0.8);
    plot.faclegend(d$cols,d$alleles,d$names,xlim=c(0.51,1),ylim=c(0.6,.98));
}
#################################################
plotparents <- function(data,males,females)
{
    m <- convert.data(as.matrix(males),data$names,data$alleles);
    f <- convert.data(as.matrix(females),data$names,data$alleles);
    plot(1,1,xlim=c(0,1),ylim=c(0,1),type="n",bty="n",xaxt="n"
	    ,yaxt="n",xlab="",ylab="");
    text(0.5,1,"Male Adults",cex=1.2);
    plot.factor.grid(m,data$cols
                     ,rep(1:length(data$alleles),rep(2,length(data$alleles)))
                     ,xlim=c(0,1),ylim=c(.6,.98),add=T);
    text(0.5,0.4,"Female Adults",cex=1.2);
    plot.factor.grid(f,data$cols
                     ,rep(1:length(data$alleles),rep(2,length(data$alleles)))
                     ,xlim=c(0,1),ylim=c(0,.38),add=T);
  }
#################################################
readstring <- function(data,breeding)
{
    a <- as.matrix(read.table(data));
    b <- as.matrix(read.table(breeding));
    d <- list(data=a,breeding=b);
    class(d) <- "parentagedata"
    d;
}
##################################################
plot.factor.grid <- function(data,colours,usecols,xlim=c(0,1),ylim=c(0,1),add=T)
#  a function to plot a grid of factors
#  using the colours in the ith row of colours for col i
{
    xgap <- xlim[2]-xlim[1];
    ygap <- ylim[2]-ylim[1];
    n <- nrow(data);
    cols <- ncol(data);
    if (!add)  plot(xlim,ylim,type="n",xlim=xlim,ylim=ylim,bty="n",xaxt="n"
	    ,yaxt="n",xlab="",ylab="");
    for (i in 1:cols) {
	yt <- ylim[2]-ygap*(1:n)/n;
	yb <- ylim[2]-ygap*(0:(n-1))/n;
	xl <- xlim[1]+xgap*rep((i-1),n)/cols;
	xr <- xlim[1]+xgap*rep(i,n)/cols;   
	rect(xl,yb,xr,yt,col=colours[usecols[i],as.numeric(data[,i])]);
    }
    ylab <- seq(5,n,by=5);
    yt <- ylim[2]-ygap*ylab/n;
    yb <- ylim[2]-ygap*(ylab-1)/n;
    xl <- rep(xlim[1],length(ylab));
    xr <- rep(xlim[2],length(ylab))
    rect(xl,yb,xr,yt,border="grey");
    rect(xlim[1],ylim[1],xlim[2],ylim[2]);
    y <- ylim[2]-ygap*(ylab-0.7)/n;
    x <- rep(xlim[1]-0.02,length(y));
    text(x,y,ylab,cex=0.6);
    text(xlim[1],ylim[2],"n",cex=0.8,offset=0.2,pos=2);
    
    lines(rep(xlim[1],2),ylim,lwd=2);
    for (i in 1:(cols/2)) 
      lines(rep(xlim[1]+i*xgap*2/cols,2),ylim,lwd=2);
}
##############################################################################
plot.faclegend <- function(colours,n,names,xlim=c(0,1),ylim=c(0,1))
{
    xgap <- xlim[2]-xlim[1];
    ygap <- ylim[2]-ylim[1];
    rows <- nrow(colours);
    mx <- max(n);

    for (i in 1:rows) {
	yt <- ylim[1]+ygap*(1:n[i])/mx;
	yb <- ylim[1]+ygap*(0:(n[i]-1))/mx;
	xl <- xlim[1]+xgap*rep((i-1),(n[i]-1))/rows;
	xr <- xlim[1]+xgap*rep(i,(n[i]-1))/rows;   
	rect(xl,yb,xr,yt,col=colours[i,]);
	nam <- names[i,1:n[i]];
	nam[nam=="-1"] <- "MISS";
	nam[nam=="999"] <- "ELSE";
	text((xl+xr)/2,(yt+yb)/2,nam,pos=NULL,cex=0.8,font=2)
    }
}
###############################################################################
plot.freqrows <- function(p,colours,alleles,xlim=c(0,1),ylim=c(0,1)
                          ,newplot=T,trans=T)
{
    xgap <- xlim[2]-xlim[1];
    ygap <- ylim[2]-ylim[1];
    rows <- nrow(p);

    if (trans) {
	for (i in 1:rows) {
	    yt <- ylim[1]+ygap*cumsum(p[i,1:(alleles[i]-1)]);
	    yb <- ylim[1]+ygap*cumsum(c(0,p[i,1:(alleles[i]-2)]));
	    xl <- xlim[1]+xgap*rep((i-1),(alleles[i]-1))/rows;
	    xr <- xlim[1]+xgap*rep(i,(alleles[i]-1))/rows;   
	    rect(xl,yb,xr,yt,col=colours[i,]);
	}
    }
  }
##############################################################################

##############################################################################
plot.shared <- function(fathers,mothers,n=10000,cls=T,levels=10)
{
    if (missing(mothers)) {
	mothers <- fathers;
	tit <- "Shared Fathers";
    } else if (missing(fathers)) {
	tit <- "Shared Mothers";
	fathers <- mothers;
    } else 
	tit <- "Shared Parents (Fathers over Diagonal)";
    l <- ncol(fathers);
    x <- 1:l;
    u <- outer(1:l,1:l,FUN=">");
    fathers[u] <- mothers[u];
    fathers <- fathers/n;
    par(mar=c(2,3,1,1))
    mat <- matrix(c(1,2),1,2,T)
    layout(mat,c(3,1), 3, T);
    if (cls==T) 
      cls <- terrain.colors(levels)
    else cls <- gray(levels:1/levels);
    image(x,x,fathers,bty="o",xaxs="i",yaxs="i",col=cls,
          plot.title=title(main=tit),
          plot.axes={ 
            axis(1,at=seq(5,l,5))
            axis(2,at=seq(5,l,5))},
          )	
    box();
    par(mar=c(2,1,1,3))
    plot(1,1,xaxs="i",yaxs="i",xlim=c(0,1),ylim=c(0,1),type="n",bty="n"
         ,xaxt="n",yaxt="n",xlab="",ylab="")
    axis(4, seq(0, 1, by = 0.1));
    rect(rep(0,levels),seq(0,(levels-1)/levels,1/levels)
         ,rep(1,levels),seq(1/levels,1,1/levels),col=cls,border=cls
         )
    rect(0,0,1,1)
    title(main="Proportion");
  }
###############################################################################
generate.alldata <- function(all)
{
#  a function to take background and data
#  and to convert the background frequencies
#  and to produce a set of colours for each with 
#  the convention that black is missing (for data)
#  or unobserved in the data (for background)
    alleles <- count.alleles(all$data);
    names <- allele.names(all$data,alleles);
    data <- convert.data(all$data,names,alleles);
    back <- convert.data(all$breeding,names,alleles);
    cols <- allele.colours(names,alleles);
    p <- calc.allelefreq(back,alleles);
    d <- list(alleles=alleles,names=names,data=data,p=p,cols=cols);
    all.warn(d);
    class(d) <- "plotinfo";
    d;
}
###########################################################################
count.alleles <- function(data)
{
    l <- ncol(data)/2;
    alleles <- numeric(l);
    for (i in 1:l) {
        use <- c((2*i-1),(2*i));
        d <- factor(data[,use],exclude=-1);
        alleles[i] <- length(levels(d))+2;
    }
    alleles
}
###############################################################################	
allele.names <- function(data,alleles)
{
    l <- length(alleles);
    mx <- max(alleles);
    names <- matrix(numeric(l*mx),nrow=l);
    for (i in 1:l) {
	use <- c((2*i-1),(2*i));
	d <- c("-1",levels(factor(data[,use],exclude=-1)),"999");
	names[i,] <- pad(d,mx,"0");
    }
    names;
}
#################################################
pad <- function(a,len,what) 
{
    l <- length(a);
    if (l>len) warning(paste("a is longer than",len))
    else if (l==len) return(a)
    else return(c(a,rep(what,len-l)))
}
#################################################
convert.data <- function(data,names,alleles)
{
    l <- ncol(data);
    mx <- max(alleles);
    cnv <- data;
    for (i in 1:l) {
	use <- as.integer((i+1)/2);
	f <- factor(data[,i],levels=names[use,]);  
	d <- as.numeric(f);
	d[is.na(d)] <- alleles[use];
	cnv[,i] <- d;
    }
    cnv;
}
#################################################
allele.colours <- function(names,alleles)
{
    cls <- names; # to give a matrix of the correct size
    for (i in 1:length(alleles)) {
      cls[i,] <- pad(c("lightgrey",rainbow(alleles[i]-2))
                     ,max(alleles),"white");
    }
    cls;
}
#################################################
calc.allelefreq <- function(data,alleles)
{
    l <- length(alleles);
    mx <- max(alleles)-1;
    p <- matrix(numeric(mx*l),nrow=l);
    for (i in 1:l) {
	use <- c((2*i-1),(2*i));
        d <- factor(data[,use]);
	t <- tabulate(d);
	p[i,] <- pad(t[-1]/sum(t[-1]),mx,0.0);
    }
    p
}
#################################################
all.warn <- function(d)
{
    mess <- "";
    mx <- max(d$alleles);
    for (i in 1:length(d$alleles)){
	prob <- d$p[i,2:(d$alleles[i]-1)]<=0.0;
	if (sum(prob)>0) {
	    ms <- paste("allele",d$names[i,pad(c(F,prob),mx,F)],"is not in background");
	    ms <- paste("locus",i,ms,"\n",collapse=" ");
	    mess <- paste(mess,ms,collapse="\n");
	}
    }
    if (nchar(mess)>0) cat(paste("warning \n",mess));
}

########################################################################
plot.plotinfo <- function(d,...)
{
    plot(1,1,xlim=c(0,1),ylim=c(0,1),type="n",bty="n",xaxt="n"
	    ,yaxt="n",xlab="",ylab="",...);
    plot.factor.grid(d$data,d$cols
                     ,rep(1:length(d$alleles),rep(2,length(d$alleles)))
                     ,xlim=c(0,0.5),ylim=c(0,.98),add=T)
    plot.freqrows(d$p,d$cols[,-1],d$alleles,xlim=c(0.51,1)
                  ,ylim=c(0,0.5),newplot=F,trans=T)
    text(0.75,0.54,"Background Frequencies",cex=1.2);
    text(0.25,1.02,"Locus",cex=1.2);
    l <- length(d$alleles);
    text(seq(0.5/(2*l),0.5-0.5/(2*l),0.5/l),rep(0.995,l),1:l,cex=0.8);
    text(0.75,1.02,"Key",cex=1.2);
    text(seq(0.5+0.5/(2*l),1-0.5/(2*l),0.5/l),rep(0.995,l),1:l,cex=0.8);
    plot.faclegend(d$cols,d$alleles,d$names,xlim=c(0.51,1),ylim=c(0.6,.98));
}
########################################################################
